unit uOptionsUI;
{$INCLUDE 'VirusTotal.inc'}

interface

uses
  SysUtils,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uOptions;

type
  TOptionsDialog = class(TOkCancelDialog)
  private
    fOptions: TOptions;
    fLabelApiKey: TFarDialog.TLabel;
    fButtonApiKey: TFarDialog.TButton;
    fEditApiKey: TFarDialog.TEdit;
    fLabelMaxFileSize: TFarDialog.TLabel;
    fEditMaxFileSize: TFarDialog.TEdit;
    fLabelInitialScanTime: TFarDialog.TLabel;
    fEditInitialScanTime: TFarDialog.TEdit;
    fLabelDelayBetweenReportChecks: TFarDialog.TLabel;
    fEditDelayBetweenReportChecks: TFarDialog.TEdit;
    fLabelCommandLinePrefix: TFarDialog.TLabel;
    fEditCommandLinePrefix: TFarDialog.TEdit;
    fCheckboxUseRescanAPI: TFarDialog.TCheckbox;
  protected
    procedure Clear; override;
    procedure CreateUserControls; override;
    procedure SetSize(Width, Height: integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    procedure ButtonApiKeyClick(Sender: TFarDialog.TButton; var StopProcessing: boolean); virtual;
    property LabelApiKey: TFarDialog.TLabel read fLabelApiKey;
    property ButtonApiKey: TFarDialog.TButton read fButtonApiKey;
    property EditApiKey: TFarDialog.TEdit read fEditApiKey;
    property LabelMaxFileSize: TFarDialog.TLabel read fLabelMaxFileSize;
    property EditMaxFileSize: TFarDialog.TEdit read fEditMaxFileSize;
    property LabelInitialScanTime: TFarDialog.TLabel read fLabelInitialScanTime;
    property EditInitialScanTime: TFarDialog.TEdit read fEditInitialScanTime;
    property LabelDelayBetweenReportChecks: TFarDialog.TLabel read fLabelDelayBetweenReportChecks;
    property EditDelayBetweenReportChecks: TFarDialog.TEdit read fEditDelayBetweenReportChecks;
    property LabelCommandLinePrefix: TFarDialog.TLabel read fLabelCommandLinePrefix;
    property EditCommandLinePrefix: TFarDialog.TEdit read fEditCommandLinePrefix;
    property CheckboxUseRescanAPI: TFarDialog.TCheckbox read fCheckboxUseRescanAPI;
  public
    constructor Create;
    destructor Destroy; override;
    function Execute: boolean;
    property Options: TOptions read fOptions write fOptions;
  end;

implementation

uses
  ShellApi, Windows,
  uMessages, uConsts;

{ TOptionsDialog }

procedure TOptionsDialog.ButtonApiKeyClick(Sender: TFarDialog.TButton; var StopProcessing: boolean);
begin
  ShellExecute(0, 'open', 'https://www.virustotal.com/gui/join-us', nil, nil, SW_SHOWNORMAL);
  StopProcessing := True;
end;

procedure TOptionsDialog.Clear;
begin
  inherited;
  fLabelApiKey := nil;
  fButtonApiKey := nil;
  fEditApiKey := nil;
  fLabelMaxFileSize := nil;
  fEditMaxFileSize := nil;
  fLabelInitialScanTime := nil;
  fEditInitialScanTime := nil;
  fLabelDelayBetweenReportChecks := nil;
  fEditDelayBetweenReportChecks := nil;
  fLabelCommandLinePrefix := nil;
  fEditCommandLinePrefix := nil;
  fCheckboxUseRescanAPI := nil;
end;

constructor TOptionsDialog.Create;
begin
  inherited Create(DIALOG_OPTIONS_GUID);
end;

procedure TOptionsDialog.CreateUserControls;
begin
  inherited;
  fLabelApiKey := AddLabel(0, 0, 0, GetMsg(MLabelApiKey));
  fButtonApiKey := AddButton(0, 0, GetMsg(MButtonApiKey));
  fButtonApiKey.CenterGroup := False;
  fButtonApiKey.NoClose := True;
  fButtonApiKey.OnClick := ButtonApiKeyClick;
  fEditApiKey := AddEdit(0, 0, 0, '');
  fLabelMaxFileSize := AddLabel(0, 0, 0, GetMsg(MLabelMaxFileSize));
  fEditMaxFileSize := AddEdit(0, 0, 0, '');
  fLabelInitialScanTime := AddLabel(0, 0, 0, GetMsg(MLabelInitialScanTime));
  fEditInitialScanTime := AddEdit(0, 0, 0, '');
  fLabelDelayBetweenReportChecks := AddLabel(0, 0, 0, GetMsg(MLabelDelayBetweenReportChecks));
  fEditDelayBetweenReportChecks := AddEdit(0, 0, 0, '');
  fLabelCommandLinePrefix := AddLabel(0, 0, 0, GetMsg(MLabelCommandLinePrefix));
  fEditCommandLinePrefix := AddEdit(0, 0, 0, '');
  fCheckboxUseRescanAPI := AddCheckbox(0, 0, GetMsg(MUseRescanAPI), False);
end;

destructor TOptionsDialog.Destroy;
begin
  inherited;
end;

function TOptionsDialog.Execute: boolean;
begin
  Result := False;
  EditApiKey.Text := Options.ApiKey;
  EditMaxFileSize.Text := Options.MaxFileSizeStr;
  EditInitialScanTime.Text := IntToStr(Options.InitialScanTime);
  EditDelayBetweenReportChecks.Text := IntToStr(Options.DelayBetweenReportChecks);
  EditCommandLinePrefix.Text := Options.CommandLinePrefix;
  CheckboxUseRescanAPI.Checked := Options.UseRescanAPI;
  ActiveControl := EditApiKey;
  if inherited Execute then
  begin
    Options.ApiKey := EditApiKey.Text;
    Options.MaxFileSizeStr := EditMaxFileSize.Text;
    Options.InitialScanTime := StrToIntDef(EditInitialScanTime.Text, Options.InitialScanTime);
    Options.DelayBetweenReportChecks := StrToIntDef(EditDelayBetweenReportChecks.Text, Options.DelayBetweenReportChecks);
    Options.CommandLinePrefix := EditCommandLinePrefix.Text;
    Options.UseRescanAPI := CheckboxUseRescanAPI.Checked;
    Result := True;
  end;
end;

procedure TOptionsDialog.SetSize(Width, Height: integer);
begin
  inherited;
  LabelApiKey.AlignTo(Frame, 2, 1, -4, 0);
  ButtonApiKey.Top := LabelApiKey.Top;
  ButtonApiKey.Width := Length(ButtonApiKey.Text) + 4;
  ButtonApiKey.Left := LabelApiKey.Left + LabelApiKey.Width - ButtonApiKey.Width;
  EditApiKey.AlignTo(LabelApiKey, 0, 1);
  LabelMaxFileSize.AlignTo(EditApiKey, 0, 1);
  EditMaxFileSize.AlignTo(LabelMaxFileSize, 0, 1);
  LabelInitialScanTime.AlignTo(EditMaxFileSize, 0, 1);
  EditInitialScanTime.AlignTo(LabelInitialScanTime, 0, 1);
  LabelDelayBetweenReportChecks.AlignTo(EditInitialScanTime, 0, 1);
  EditDelayBetweenReportChecks.AlignTo(LabelDelayBetweenReportChecks, 0, 1);
  LabelCommandLinePrefix.AlignTo(EditDelayBetweenReportChecks, 0, 1);
  EditCommandLinePrefix.AlignTo(LabelCommandLinePrefix, 0, 1);
  CheckboxUseRescanAPI.AlignTo(EditCommandLinePrefix, 0, 1);
end;

procedure TOptionsDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  //Width := 40;
  Height := 17;
end;

end.
